########################################################################
#                                                                      #
# Tkbb - check book accounting program written in "pure" tcl/tk        #
#        The code is U.S. specific, no attempt has been made to        #
#        internationalize the display or verification of currency.     #
#        Tcl/Tk code is so easy to modify is should be fairly easy to  #
#        customize for individual use.                                 #
#                                                                      #
# I must express my gratitude to Bryan Oakley, oakley@channelpoint.com #
# for his excellent combobox widgets.  My thanks also goes to Simon    #
# Simon Edwards, Proprius Consulting Ltd for his datechooser widget.   #
# Tkbb would not be possible without the silent contributions of these #
# open-source authors.                                                 #
#                                                                      #
# This code is freely distributable without restriction, but is        #
# provided as-is with no waranty expressed or implied.                 #
#                                                                      #
# Copyright (c) 1996-2001, Dave Freese                                 #
# All Rights Reservered                                                #
#                                                                      #
# Dave Freese                                                          #
# dave@darkmatterfactory.com                                           #
#                                                                      #
# tkbb v1.0 August 14, 2001                                            #
#                                                                      #
########################################################################

lappend auto_path ./

source megawidget.tcl
catch {namespace import megawidget::*}
source calendar.tcl
catch {namespace import calendar::*}
source dateentry.tcl
catch {namespace import dateentry::*}
source combobox.tcl
catch {namespace import combobox::*}

source  tkbb.dialog.tcl
source  tkbb.scrolled_list.tcl
source  tkbb.bitmaps.tcl

global  chkBalance; 
global  chkCredit; 
global  chkDate; 
global  chkDebit; 
global  chkMemo; 
global  chkNbr; 
global  chkPayee; 
global  chkReconcile;

global  lstBalances;
global  lstMemos
global  lstPayees
global  lstXacts;

global  acctBalance; 
global  stmtBalance;

global  dirty; 
global  ext_type;
global  fields;
global  fname; 
global  lastchecknbr;
global  num
global  myappname
global  scrollfont
global  selectedRecord;
global  state; 
global  txtBtnNew; 
global  txtBtnUpdate;

global  wdgNbr;
global  wdgPayee;
global  wdgMemo;
global  wdgBtnDel;
global  wdgChkList;

if {$tcl_platform(platform) == "unix"} {
  set scrollfont {courier 12}
  } else {
  set scrollfont {courier 8}
}

set lastchecknbr 0
set fields "REC|NUMBER|DATE|DESC|MEMO|DEBIT|CREDIT"
set ext_type { {"tkbb data file" {".tkb"}} }
set num 0
set myappname ""

proc loading_splash_on {filename} {
  global dlgLoading progressleft progressright

  set dlgLoading [toplevel .wdlg -bd 3 -relief sunken ]
  set sw [winfo screenwidth .]
  set sh [winfo screenheight .]

  wm overrideredirect .wdlg 1

  set spwidth 400
  set spheight 220
  
  set x [expr {($sw - $spwidth) / 2}]
  set y [expr {($sh - $spheight) / 2}]
  
  wm geometry .wdlg +$x+$y

  frame .wdlg.f1 -width $spwidth -height 180
  if {[catch {image create photo "tkbb" -file "tkbb.gif"} result]} {
  } else {
    label .wdlg.f1.img2 -image tkbb -bd 1 -relief groove
    pack .wdlg.f1.img2 -side left -expand 1 -fill both
  }
  pack .wdlg.f1 -side top
  label .wdlg.lbl -text "Loading: $filename" -font {courier 14} \
    -relief flat \
    -anchor center
  pack .wdlg.lbl -side top -fill x
  frame .wdlg.f -width 200 -height 10
  set progressleft [frame .wdlg.f.p1 -width 1 -height 10 -bg blue]
  set progressright [frame .wdlg.f.p2 -width 199 -height 10]
  pack  .wdlg.f.p2 -side right
  pack  .wdlg.f.p1 -side left
  pack  .wdlg.f -side top
  frame .wdlg.filler -height 5
  pack  .wdlg.filler -side top -fill x
  wm deiconify .wdlg
}

proc progress {n max} {
  global dlgLoading progressright progressleft
  if {[winfo exists $dlgLoading]} {
    set sizeleft [expr {$n * 400 / $max}]
    set sizeright [expr {400 - $sizeleft}]
    $progressleft  config -width $sizeleft
    $progressright config -width $sizeright
    update idletasks
  }
}

proc loading_splash_off {} {
  global dlgLoading
  destroy $dlgLoading
  wm deiconify .
  update idletasks
}

proc tkbb_splash_on {} { 
  global tcl_platform splash_win
  set splash_win [toplevel .x -bd 3 -relief raised]
  set sw [winfo screenwidth .]
  set sh [winfo screenheight .]
  set spheight 180
  set spwidth  400
  wm overrideredirect .x 1
  frame .x.f1 -width $spwidth -height $spheight
  if {[catch {image create photo "tkbb" -file "tkbb.gif"} result]} {
  } else {
    label .x.f1.img2 -image tkbb -bd 1 -relief flat
    pack .x.f1.img2 -side left -expand 1 -fill both
  }
  pack .x.f1 -side top
  set x [expr {($sw - $spwidth)/2}]
  set y [expr {($sh - $spheight)/2}]
  wm geometry .x +$x+$y
  wm withdraw .
  wm deiconify .x
}

proc tkbb_splash_off {} {
  global splash_win
  destroy $splash_win
  wm deiconify .
}

proc Title {name} {
  set title "Tk Bankbook  "
  if {[string length $name] > 0} {
    append title [file tail $name]
  }
  wm title . $title
}

proc CancelUpdate {} {
  global state
  if {$state == "view"} {
    Record_update
  }
  if {$state == "enter"} {
    set state view
    ShowState
  }
  DisplaySelected
}

proc validAmount amount {
  if {[catch {set amt [format "%9.2f" $amount]} result]} {
    set amt 0.00
  }
  return $amt
}

proc validDate Date {
  set a [split $Date /]
  if {[catch {
        set mm [format "%d" [string trimleft [lindex $a 0] 0] ]
        set dd [format "%d" [string trimleft [lindex $a 1] 0] ]
        set yy [format "%d" [string trim [lindex $a 2] ] ]
      } result]} {
    set t [clock seconds]
    set mm [clock $t -format "%m"]
    set dd [clock $t -format "%d"]
    set yy [clock $t -format "%Y"]
  }
  if {$yy < 1900} {
    if {$yy < 50} {
      set yy [expr {$yy + 2000}]
    } else {
      set yy [expr {$yy + 1900}]
    }
  } 
  return [format "%02d/%02d/%4d" $mm $dd $yy]
}

proc validateFields {} {
  global chkNbr chkDate chkPayee chkMemo chkCredit chkDebit
  set chkDebit  [validAmount $chkDebit]
  set chkCredit [validAmount $chkCredit]
  set chkNbr    [string trim $chkNbr]
  set chkPayee  [string trim $chkPayee]
  set chkMemo   [string trim $chkMemo]
  set chkDate   [validDate $chkDate]
}

proc DisplaySelected {} {
  global wdgChkList num selectedRecord state lstXacts
  global lstBalances chkNbr chkDate chkPayee chkMemo chkCredit 
  global chkDebit chkBalance chkReconcile

  if {$state == "enter"} return
  if { $num == 0 } return

  set chk [split [lindex $lstXacts $selectedRecord] |]
  if {[llength $chk] == 0} return

  if {[lindex $chk 0] == "X"} {
    set chkReconcile 1
  } else {
    set chkReconcile 0
  }

  set chkNbr      [lindex $chk 1]
  set chkDate     [lindex $chk 2]
  set chkPayee    [string trim [lindex $chk 3]]
  set chkMemo     [string trim [lindex $chk 4]]
  set chkDebit    [lindex $chk 5]
  set chkCredit   [lindex $chk 6]
  set chkBalance  [lindex $lstBalances $selectedRecord]
}

proc balances {from} {
  global num lstXacts lstBalances stmtBal acctBalance stmtBalance

  if {$num == 0} {
    return
  }

  if {$from == 0} {
    set balance 0
    set statement 0
  } else {
    set balance [lindex $lstBalances [expr {$from - 1}]]
    set statement [lindex $stmtBal [expr {$from - 1}]]
  } 

  set b1 {} 
  set b2 {}

  for {set n $from} {$n < $num} {incr n} {
    set chk [split [lindex $lstXacts $n] |]
    if {[llength $chk] == 0} break
    set debit [lindex $chk 5]
    set credit [lindex $chk 6]
    set balance [validAmount [expr {$balance + $credit - $debit}]]
    lappend b1 $balance
    set rec [lindex $chk 0]
    if {$rec == "X"} {
      set statement [expr {$statement + $credit - $debit}]
    }
    lappend b2 $statement
  }
  
  if {$from == 0} { 
    set lstBalances $b1
    set stmtBal $b2
  } else {
    set lstBalances [concat [lrange $lstBalances 0 [expr {$from - 1}]] $b1]
    set stmtBal [concat [lrange $stmtBal 0 [expr {$from - 1}]] $b2]
  }

  set acctBalance [validAmount $balance]
  set stmtBalance [validAmount $statement]
}

proc gotoSelectedRecord {} {
  global wdgChkList
  global selectedRecord
  $wdgChkList selection set $selectedRecord
  $wdgChkList see $selectedRecord
  DisplaySelected
}

proc GoFirst {} {
  global wdgChkList num selectedRecord

  if {$num < 1} return
  if {$selectedRecord == 0} return

  $wdgChkList selection clear $selectedRecord
  set selectedRecord 0
  gotoSelectedRecord
}

proc GoPrev {} {
  global wdgChkList num selectedRecord

  if {$num < 1} return
  if {$selectedRecord == 0} return

  $wdgChkList selection clear $selectedRecord
  incr selectedRecord -1
  gotoSelectedRecord
}

proc GoNext {} {
  global wdgChkList num selectedRecord

  if {$num < 1} return

  set l [$wdgChkList index end]
  incr l -1
  if {$selectedRecord == $l} return

  $wdgChkList selection clear $selectedRecord
  incr selectedRecord
  gotoSelectedRecord
}

proc GoLast {} {
  global wdgChkList num selectedRecord

  if {$num == 0} return
  if {$selectedRecord == [$wdgChkList index end]} return

  $wdgChkList selection clear $selectedRecord
  set selectedRecord [$wdgChkList index end]
  incr selectedRecord -1
  gotoSelectedRecord
}

proc CheckListClicked {} {
  global wdgChkList selectedRecord
  set selectedRecord [lindex [$wdgChkList curselection] 0]
  DisplaySelected
}

proc LoadRegister {at} {
  global lstXacts lstBalances wdgChkList num
  global lastchecknbr

  set RegisterList  {}
  for {set n $at} {$n < $num} {incr n} {
    set tmp [lindex $lstXacts $n]
    if {[llength $tmp] == 0} break
    set chk [split $tmp |]
    lappend RegisterList \
      [format "%1s|%-5s|%10s|%-30s|%9.2f|%9.2f|%10.2f" \
            [lindex $chk 0] \
            [lindex $chk 1] \
            [lindex $chk 2] \
            [lindex $chk 3] \
            [lindex $chk 5] \
            [lindex $chk 6] \
            [lindex $lstBalances $n]]
    set chknbr [lindex $chk 1]
    if {[catch {expr {$chknbr + 0}} result] == 0} {
      if {$chknbr > $lastchecknbr} {
        set lastchecknbr $chknbr
      }
    }
    progress $n $num
  }

# loading all of the items into the list widget at one time
# significantly improves the performance

  $wdgChkList delete $at end
  eval {$wdgChkList insert end} $RegisterList
  
}

proc LoadlstPayees {} {
  global wdgPayee lstPayees
  $wdgPayee list delete 0 end
  eval {$wdgPayee list insert end} $lstPayees
}

proc LoadlstMemos {} {
  global wdgMemo lstMemos
  $wdgMemo list delete 0 end
  eval {$wdgMemo list insert end} $lstMemos
  
}

proc File_read {} {
  global num lstXacts fname datFile payeeFile categoryFile
  global lstPayees lstMemos
  global wdgPayee wdgMemo
  global myappname

  set fname [file root $fname]
  Title $fname

  set datFile $fname  
  append datFile ".tkb"

  set payeeFile $fname
  append payeeFile ".pay"

  set categoryFile $fname
  append categoryFile ".cat"  

  if {[file exists $datFile]} {
    set stream [open $datFile r]
    set data [read $stream]
    close $stream
    set lstXacts [split $data \n]
    if {[string first REC [lindex $lstXacts 0]] > -1} {
      set lstXacts [lreplace $lstXacts 0 0]
    }
    if {[llength [lindex $lstXacts end]] == 0} {
      set lstXacts [lreplace $lstXacts end end]
    }  
    set num [llength $lstXacts]
  } else {
    set num 0 
  }

  if {[file exists $payeeFile]} {
    set stream [open $payeeFile]
    set data [read $stream]
    close $stream
    set lstPayees [split $data "\n"]
    if {[llength [lindex $lstPayees end]] == 0} {
      set lstPayees [lreplace $lstPayees end end]
    }
  } else {
    set lstPayees {}
    foreach line $lstXacts {
      set payee [lindex [split $line |] 3]
      if {[string length $payee] > 0} {
        if {[lsearch $lstPayees $payee] == -1} {lappend lstPayees $payee}
      }
    }
    set lstPayees [lsort -dictionary $lstPayees]
  }
    
  if {[file exists $categoryFile]} {
    set stream [open $categoryFile]
    set data [read $stream]
    close $stream
    set lstMemos [split $data \n]
    if {[llength [lindex $lstMemos end]] == 0} {
      set lstMemos [lreplace $lstMemos end end]
    }  
  } else {
    set lstMemos {}
    foreach line $lstXacts {
      set cat   [lindex [split $line |] 4]
      if {[string length $cat] > 0} {
        if {[lsearch $lstMemos $cat] == -1} {lappend lstMemos $cat}
      }
    }
    set lstMemos [lsort -dictionary $lstMemos]
  }
  
  LoadlstPayees
  LoadlstMemos
  
  set myappname "tkbb.$fname"
  tk appname $myappname
}

proc File_open {} {
  global wdgChkList lstBalances lstXacts dirty
  global selectedRecord num

  set lstBalances {}
  set lstXacts {}

  File_read
  set dirty 0

  balances 0
  LoadRegister 0

  set selectedRecord [expr {$num - 1}]
}

proc mFile_open_cb {} {
  global dirty fname ext_type
  
  if {$dirty == 1} {
    set choice [tk_messageBox \
                -type yesno -default yes \
                -message "Save Changes?" \
                -icon question]
    if {$choice == "yes"} mbFile_save_cb
  }
  update idletasks
  
  set fname ""
  set fname [tk_getOpenFile \
             -title "Open Bankbook File" \
             -filetypes $ext_type]

  if {[string length $fname] == 0} return

  loading_splash_on [file tail $fname]
  File_open
  loading_splash_off
  gotoSelectedRecord
}

proc UpdateCombo {} {
  global lstPayees lstMemos
  global wdgPayee wdgMemo
  global chkPayee chkMemo

  if {[string length $chkPayee] > 0} {
    if {[lsearch $lstPayees $chkPayee] == -1} {
      lappend lstPayees $chkPayee
      set lstPayees [lsort -dictionary $lstPayees]
      $wdgPayee list delete 0 end
      eval {$wdgPayee list insert end} $lstPayees
    }
  } 

  if {[string length $chkMemo] > 0} {
    if {[lsearch $lstMemos $chkMemo] == -1} {
      lappend lstMemos $chkMemo
      set lstMemos [lsort -dictionary $lstMemos]
      $wdgMemo list delete 0 end
      eval {$wdgMemo list insert end} $lstMemos
    }
  }
}

proc save_file {} {
  global fname lstXacts lstPayees lstMemos fields dirty

  set fname [file root $fname]
  set datFile $fname  
  append datFile ".tkb"

  set payeeFile $fname
  append payeeFile ".pay"

  set categoryFile $fname
  append categoryFile ".cat"  

  set data [join $lstXacts \n]
  set stream [open $datFile w]

  puts $stream $fields
  puts -nonewline $stream $data
  close $stream

  set data [join $lstPayees \n]
  set stream [open $payeeFile w]
  puts -nonewline $stream $data
  close $stream

  set data [join $lstMemos \n]
  set stream [open $categoryFile w]
  puts -nonewline $stream $data
  close $stream

  set dirty 0
}

proc mbFile_save_as_cb {} {
  global fname ext_type

  set fname [tk_getSaveFile \
             -title "Save Bankbook File As" \
             -filetypes $ext_type]

  if {[string length $fname] == 0} return

  Title $fname
  save_file
}

proc mbFile_save_cb {} {
  global fname

  if {[string length $fname] == 0} {
    mbFile_save_as_cb
    return
  }
  save_file
}

proc mExit_cb {} {
  global dirty

  if {$dirty == 1} {
    set choice [tk_messageBox \
                -type yesno -default yes \
                -message "Save Changes?" \
                -icon question]
    if {$choice == "yes"} mbFile_save_cb
  }
  exit
}

proc NewSave {} {
  global wdgNbr chkNbr chkDate chkPayee chkMemo chkCredit 
  global chkDebit chkBalance chkReconcile state
  global lastchecknbr

  if {$state == "view"} {
    set chkNbr [expr {$lastchecknbr + 1}]
    set chkDate [clock format [clock seconds] -format "%m/%d/%Y"]
    set chkPayee ""
    set chkMemo ""
    set chkCredit ""
    set chkDebit ""
    set chkBalance ""
    set chkReconcile ""
    set state enter
    focus $wdgNbr
    ShowState
    return
  }
  if {$state == "enter"} {
    Record_add
    set state view
    ShowState
    DisplaySelected
    return
  }
}

proc Record_add {} {
  global wdgChkList lstXacts lstBalances num dirty
  global chkNbr chkDate chkPayee chkMemo chkCredit chkDebit 
  global chkBalance chkReconcile 
  global selectedRecord

  if {$chkReconcile == 1} {
    set r X
  } else {
    set r ""
  } 

  validateFields

  set line [join [list $r $chkNbr $chkDate $chkPayee $chkMemo $chkDebit $chkCredit] |]

  set chk ""
  for {set n [expr {$num - 1}]} {$n > -1} {incr n -1} {
    set tmp [lindex $lstXacts $n]
    if {[llength $tmp] == 0} {
      break
    }
    set chk [split $tmp |]
    set c1 [compdates [string trim [lindex $chk 2]] $chkDate]
    if {$c1 == -1} break
    if {$c1 == 0} {
      set c2 [string compare [string trim [lindex $chk 1]] $chkNbr]
      if {$c2 < 1} break
    }
  }

  incr n
  set lstXacts [linsert $lstXacts $n $line]
  set lstBalances [linsert $lstBalances $n 0]

  set num [llength $lstXacts]
  balances $n

  LoadRegister $n

  $wdgChkList selection clear $selectedRecord
  set selectedRecord $n
  gotoSelectedRecord
  
  UpdateCombo
  set dirty 1
}

proc Record_delete {} {
  global wdgChkList lstXacts num dirty
  global chkNbr chkDate chkPayee chkMemo chkCredit chkDebit 
  global chkBalance chkReconcile 
  global selectedRecord

  if {$num == 0} return

  set choice [tk_messageBox -type yesno -default no -message "Confirm Delete" \
              -icon question]
  if {$choice == "no"} return            

  set n $selectedRecord
  $wdgChkList selection clear $n
  set lstXacts [lreplace $lstXacts $n $n]
  $wdgChkList delete $n $n
  set num [llength $lstXacts]

  balances $n
  LoadRegister $n

  set selectedRecord [expr {$num - 1}]
  gotoSelectedRecord
  set dirty 1
}

proc Record_update {} {
  global lstXacts wdgChkList num selectedRecord

  if {$num == 0} return  

  $wdgChkList selection clear $selectedRecord
  set lstXacts [lreplace $lstXacts $selectedRecord $selectedRecord]
  $wdgChkList delete $selectedRecord $selectedRecord
  set num [llength $lstXacts]
  Record_add
}

proc ShowState {} {
  global state wdgBtnDel txtBtnUpdate txtBtnNew

  if {$state == "view"} {
    $wdgBtnDel configure -state normal
    set txtBtnUpdate Update
    set txtBtnNew New
    return
  }
  if {$state == "enter"} {
    $wdgBtnDel configure -state disabled
    set txtBtnUpdate Cancel
    set txtBtnNew Save
    return
  }
}

proc compdates {d1 d2} {
  set a [split $d1 /]
  set b [split $d2 /]
  set c [string compare [lindex $a 2] [lindex $b 2]]
  if {$c == 0} {
    set c [string compare [lindex $a 0] [lindex $b 0]]
    if {$c == 0} {
      return [string compare [lindex $a 1] [lindex $b 1]]
    }
  }
  return $c
}

proc getaccounts {} {
  global acclist myappname

  set accounts [winfo interps]
  set acclist {}
  foreach account $accounts {
    if { [string first $myappname $account] == -1 } {
      set acc [split $account .]
      if { [lindex $acc 0] == "tkbb" } {
        lappend acclist [lindex $acc 1]
      }
    }
  }
}

proc xfrOut { toAccount amount date } {
  global wdgNbr chkNbr chkDate chkPayee chkMemo chkCredit 
  global chkDebit chkBalance chkReconcile

  set chkNbr ""
  set chkDate $date
  set chkPayee "Transfered to $toAccount"
  set chkMemo "Funds Transfer"
  set chkCredit ""
  set chkDebit $amount
  set chkBalance ""
  set chkReconcile ""
  Record_add
  DisplaySelected
}

proc xfrIn { fmAccount amount date } {
  global wdgNbr chkNbr chkDate chkPayee chkMemo chkCredit 
  global chkDebit chkBalance chkReconcile

  set chkNbr ""
  set chkDate $date
  set chkPayee "Transfered from $fmAccount"
  set chkMemo "Funds Transfer"
  set chkCredit $amount
  set chkDebit ""
  set chkBalance ""
  set chkReconcile ""
  Record_add
  DisplaySelected
}

proc mbTransfer_cb {} {
  global acclist myappname scrollfont

  getaccounts
  if { [llength $acclist] == 0 } {
    tk_messageBox -type ok \
                  -message "No target accounts open" \
                  -icon warning
    return
  }

  global transfer
  set transfer(date) [clock format [clock seconds] -format "%m/%d/%Y"]
  set transfer(amount) 0
  set f .transfer
  if {[Dialog_Create $f "Money Transfer" -borderwidth 5]} {
    label $f.lbl -text "Select account"
    listbox $f.lstbox \
        -exportselection 0 \
        -selectmode single -setgrid 1 \
        -font $scrollfont \
        -width 10 -height 3
    set frm1 [frame $f.e]
    label $frm1.lbl1 -text "Amount"
    entry $frm1.amount -textvariable transfer(amount)
    label $frm1.lbl2 -text "Date  "
    entry $frm1.date -textvariable transfer(date)
    grid $frm1.lbl1 $frm1.amount
    grid $frm1.lbl2 $frm1.date
    set b [frame $f.buttons]
    pack $f.lbl $f.lstbox $f.e  $f.buttons -side top -fill x
    button $b.ok -text OK -command {set transfer(ok) 1}
    button $b.cancel -text Cancel -command {set transfer(ok) 0}
    pack $b.ok -side left
    pack $b.cancel -side right
  }

  $f.lstbox delete 0 end
  foreach acc $acclist {
    $f.lstbox insert end [file tail $acc]
  } 

  set transfer(ok) 0
  Dialog_Wait $f transfer(ok) $f.lstbox

  if { $transfer(ok) } {
    set selected [$f.lstbox curselection]
    set transfer(amount) [validAmount $transfer(amount)]
    set transfer(date) [validDate $transfer(date)]
    if {[llength $selected] > 0} {
      set n [lindex $selected 0]
      set myname [lindex [split $myappname .] 1]
      send tkbb.[lindex $acclist $n] \
        [list receivetransfer $myname $transfer(amount) $transfer(date)]
      xfrOut [file tail [lindex $acclist $n]] $transfer(amount) $transfer(date) 
    }
  }

  Dialog_Dismiss $f
}

proc receivetransfer {sendingaccount amount date} {
  global myappname
  xfrIn [file tail $sendingaccount] $amount $date
}

proc LoadRecLists {} {
  global debitList creditList 
  global debitNums creditNums
  global lstXacts num

  set debitNums {}
  set creditNums {}

  $debitList delete 0 end
  $creditList delete 0 end

  for {set n 0} {$n < $num} {incr n} {
    set tmp [lindex $lstXacts $n]
    if {[llength $tmp] == 0} break
    set chk [split $tmp |]
    if {[lindex $chk 0] != "X"} {
      if {[lindex $chk 5] > 0} {
        $debitList insert end \
          [format "%1s|%-5s|%10s|%8.2f|%-s" \
            [lindex $chk 0] [lindex $chk 1] [lindex $chk 2] [lindex $chk 5] \
            [string trim [lindex $chk 3]]]
        lappend debitNums $n
      } else {
      $creditList insert end \
          [format "%1s|%-5s|%10s|%8.2f|%-s" \
            [lindex $chk 0] [lindex $chk 1] [lindex $chk 2] [lindex $chk 6] \
            [string trim [lindex $chk 3]]]
        lappend creditNums $n
      }
    }
  }
}

proc modDebit {} {
  global debitList recBalance

  set n [lindex [$debitList curselection] 0]
  if {$n == ""} return

  set chk [split [$debitList get $n] |]

  if {[lindex $chk 0] == "X"} {
    set chk [lreplace $chk 0 0 " "]
    set debit [lindex $chk 3]
    set recBalance [expr {$recBalance + $debit}]
  } else {
    set chk [lreplace $chk 0 0 "X"]
    set debit [lindex $chk 3]
    set recBalance [expr {$recBalance - $debit}]
  }
  
  $debitList delete $n $n
  $debitList insert $n \
    [format "%1s|%4s|%10s|%8.2f|%-s" \
      [lindex $chk 0] [lindex $chk 1] [lindex $chk 2] \
      [lindex $chk 3] [lindex $chk 4]]
}

proc modCredit {} {
  global creditList recBalance

  set n [lindex [$creditList curselection] 0]
  if {$n == ""} return

  set chk [split [$creditList get $n] |]

  if {[lindex $chk 0] == "X"} {
    set chk [lreplace $chk 0 0 " "]
    set credit [lindex $chk 3]
    set recBalance [expr {$recBalance - $credit}]
  } else {
    set chk [lreplace $chk 0 0 "X"]
    set credit [lindex $chk 3]
    set recBalance [expr {$recBalance + $credit}]
  }
  
  $creditList delete $n $n
  $creditList insert $n \
    [format "%1s|%4s|%10s|%8.2f|%-s" \
      [lindex $chk 0] [lindex $chk 1] [lindex $chk 2] \
      [lindex $chk 3] [lindex $chk 4]]
}

proc ReconcileUpdate {} {
  global debitList creditList debitNums creditNums
  global lstXacts dirty wdgChkList

  set start -1

  if {[llength $debitNums] != 0} {
    set start [lindex $debitNums 0]
    set r 0
    foreach n $debitNums {
      set recline [split [$debitList get $r] |]
      if {[lindex $recline 0] == "X"} {
        set chk [split [lindex $lstXacts $n] |]
        set line [join [list "X" \
                        [lindex $chk 1] \
                        [lindex $chk 2] \
                        [lindex $chk 3] \
                        [lindex $chk 4] \
                        [lindex $chk 5] \
                        [lindex $chk 6]] |]
        set lstXacts [lreplace $lstXacts $n $n $line]
      }
      incr r
    }
  }

  if {[llength $creditNums] != 0} {
    if {$start == -1} {
      set start [lindex $creditNums 0]
    } else {
      if {[lindex $creditNums 0] < $start} {
        set start [lindex $creditNums 0]
      }
    }

    set r 0
    foreach n $creditNums {
      set recline [split [$creditList get $r] |]
      if {[lindex $recline 0] == "X"} {
        set chk [split [lindex $lstXacts $n] |]
        set line [join [list "X" \
                        [lindex $chk 1] \
                        [lindex $chk 2] \
                        [lindex $chk 3] \
                        [lindex $chk 4] \
                        [lindex $chk 5] \
                        [lindex $chk 6]] |]
        set lstXacts [lreplace $lstXacts $n $n $line]
      }
      incr r
    }
  }

  if {$start == -1} return

  balances $start
  LoadRegister $start
  gotoSelectedRecord
  set dirty 1
}

proc mbReconcile_cb {} {
  global lstXacts stmtBalance recBalance fname
  global reconcile scrollfont

  set reconcile(ok) 0

  global debitList creditList brecX brecOK

  set w .reconcile

  set rectitle "Reconcile "
  append rectitle [file tail $fname]

  if {[Dialog_Create $w $rectitle -borderwidth 2]} {
    set frm1 [frame $w.f1]
    label $frm1.lbl1 -text "Debits"
    label $frm1.lbl2 -text "Credits"
    pack $frm1.lbl1 $frm1.lbl2 -side left -expand true
    set frm2 [frame $w.f2]
    set debitList [Scrolled_Listbox $frm2.debit \
      -exportselection 0 \
      -selectmode single -setgrid 1 \
      -background #fefefe \
      -font $scrollfont \
      -width 40 -height 10 ]
    bind $debitList <ButtonRelease-1> {modDebit}
    set creditList [Scrolled_Listbox $frm2.credit \
      -exportselection 0 \
      -selectmode single -setgrid 1 \
      -background #fefefe \
      -font $scrollfont \
      -width 40 -height 10 ]
    bind $creditList <ButtonRelease-1> {modCredit}
    pack $frm2.debit $frm2.credit -side left \
      -expand true -fill both
    set frm3 [frame $w.f3]
    label $frm3.lbl -text "Statement:"
    entry $frm3.bal -textvariable recBalance \
      -background #e8deb2 -justify right \
      -width 10 \
      -state disabled
    pack $frm3.bal -side right
    pack $frm3.lbl -side right
    set frm4 [frame $w.buttons]
    set brecOK [button $frm4.ok -text OK -command {set reconcile(ok) 1} \
      -width 6]
    set brecX [button $frm4.cancel -text Cancel -command {set reconcile(ok) 0} \
      -width 6]
    grid $frm4.cancel $frm4.ok -padx 10
    pack $frm1 $frm2 $frm3 $frm4 -side top -fill x -pady 4
  }

  set recBalance $stmtBalance
  LoadRecLists
  Dialog_Wait $w reconcile(ok) $brecOK

  if { $reconcile(ok) } {
    ReconcileUpdate
  }

  Dialog_Dismiss $w
  destroy $w
}

proc MainWindow {} {
  global wdgNbr wdgPayee wdgMemo wdgBtnDel wdgChkList
  global tcl_platform scrollfont
  global bmLeftEnd bmLeftArrow bmRightArrow bmRightEnd

  set clrB01 #04969c
  set clrB02 #fefefe
  set clrF01 #000000

#  wm resizable . 0 0

  Title ""

  frame .fMenu \
    -borderwidth 2 -relief groove
  menubutton .fMenu.mFiles \
    -anchor w \
    -menu .fMenu.mFiles.m -padx 4 -pady 2 -text Files 
  menu .fMenu.mFiles.m \
       -cursor {} -tearoff 0 
  .fMenu.mFiles.m add command \
       -command mFile_open_cb \
       -label Open 
  .fMenu.mFiles.m add command \
       -command mbFile_save_cb \
       -label Save 
  .fMenu.mFiles.m add command \
       -command mbFile_save_as_cb \
       -label {Save As} 
  .fMenu.mFiles.m add separator
  .fMenu.mFiles.m add command \
      -command mExit_cb \
      -label Exit 

  button .fMenu.mbTransfer \
    -command mbTransfer_cb \
    -padx 4 -pady 2 -relief flat \
    -text Transfer

  button .fMenu.mbRec \
    -command mbReconcile_cb \
    -padx 4 -pady 2 -relief flat \
    -text Reconcile 

  frame .fEntry \
    -background $clrB01 -borderwidth 2 
  label .fEntry.lblNbr \
      -background $clrB01 -borderwidth 1 -foreground $clrF01 -text Nbr
  label .fEntry.lblDate \
      -background $clrB01 -borderwidth 1 -foreground $clrF01 -text Date 
  label .fEntry.lblPayee \
      -background $clrB01 -borderwidth 1 -foreground $clrF01 -text Payee 
  label .fEntry.lblDebit \
      -background $clrB01 -borderwidth 1 -foreground $clrF01 -text Debit 
  label .fEntry.lblCredit \
      -background $clrB01 -borderwidth 1 -foreground $clrF01 -text Credit 
  label .fEntry.lbllstBalances \
      -background $clrB01 -borderwidth 1 -foreground $clrF01 -text Balance 
  label .fEntry.lblMemo \
      -anchor e -background $clrB01 -foreground $clrF01 -borderwidth 1 \
      -text "Memo "

  set wdgNbr [entry .fEntry.entNbr \
      -background $clrB02 -justify right -textvariable chkNbr -width 6]

  dateentry .fEntry.entDate \
      -textvariable chkDate \
      -width 10 \
      -background $clrB02 \
      -dateformat "mm/dd/yyyy"

  set wdgPayee [combobox .fEntry.entPayee \
      -background $clrB02 -textvariable chkPayee -width 30] 
  entry .fEntry.entDebit \
      -background $clrB02 -justify right -textvariable chkDebit -width 10 
  entry .fEntry.entCredit \
      -background $clrB02 -justify right -textvariable chkCredit -width 10 
  entry .fEntry.entlstBalances \
      -background #e8deb2 -justify right -textvariable chkBalance -width 10 \
      -state disabled

  set wdgMemo [combobox .fEntry.entMemo \
      -background $clrB02 -textvariable chkMemo -width 30] 

  frame .fEntry.fSub1 \
      -background $clrB01 -borderwidth 2

  set wdgBtnDel [button .fEntry.fSub1.bDel \
        -command Record_delete -foreground #000000 \
        -width 8 -padx 6 -pady 2 \
        -text Delete] 
  button .fEntry.fSub1.bUpd \
        -command CancelUpdate -text Update \
        -width 8 -padx 6 -pady 2 \
        -textvariable txtBtnUpdate 
  button .fEntry.fSub1.bNew \
        -command NewSave -text New \
        -width 8 -padx 6 -pady 2 \
        -textvariable txtBtnNew 
  frame .fEntry.fSub2 \
        -background $clrB01 -borderwidth 2
  button .fEntry.fSub2.bFirst \
        -command GoFirst \
        -width 16 -height 16 \
        -image bmLeftEnd 
  button .fEntry.fSub2.bPref \
        -command GoPrev \
        -width 16 -height 16 \
        -image bmLeftArrow 
  button .fEntry.fSub2.bNext \
        -command GoNext \
        -width 16 -height 16 \
        -image bmRightArrow 
  button .fEntry.fSub2.bLast \
        -command GoLast \
        -width 16 -height 16 \
        -image bmRightEnd
  checkbutton .fEntry.cheRec \
      -activebackground $clrB01 \
      -highlightbackground $clrB01\
      -activeforeground #f4be34 \
      -background $clrB01 \
      -foreground $clrF01 \
      -text Reconciled -variable chkReconcile 

  frame .register -borderwidth 2 -relief groove

  label .register.headings \
      -font $scrollfont \
      -text "R| NBR |   DATE   |              PAYEE           |  DEBIT  | CREDIT  | BALANCE  "

  set wdgChkList [listbox .register.lstbox \
      -exportselection 0 \
      -selectmode single \
      -setgrid 1 \
      -background #fefefe \
      -font $scrollfont \
      -width 80 -height 10 \
      -yscrollcommand ".register.yscroll set"]
  scrollbar .register.yscroll -command ".register.lstbox yview" \
      -orient vertical
  bind $wdgChkList <ButtonRelease-1> {CheckListClicked}

  grid .register.headings -row 0 -column 0 -sticky nw
  grid .register.lstbox -row 1 -column 0 -sticky news
  grid .register.yscroll -row 1 -column 1 -sticky news
  grid rowconfigure .register 1 -weight 1
  grid columnconfigure .register 0 -weight 1

  frame .fStatus \
    -background $clrB01 -borderwidth 2 -height 90 -relief sunken
  label .fStatus.lblStmtBal \
      -background $clrB01 -borderwidth 1 -foreground $clrF01 \
      -text Statement: \
      -justify right
  entry .fStatus.entStmtBal \
      -background #e8deb2 -justify right -textvariable stmtBalance \
      -width 10 \
      -state disabled
  label .fStatus.lblActBal \
      -background $clrB01 -borderwidth 1 -foreground $clrF01 \
      -text "Account Balance:" \
      -justify right
  entry .fStatus.entActBal \
      -background #e8deb2 -justify right -textvariable acctBalance \
      -width 10 \
      -state disabled

  pack .fMenu \
    -in . -anchor center -expand 0 -fill x -side top 
  pack .fMenu.mFiles \
    -in .fMenu -anchor w -expand 0 -fill x -side left 
  pack .fMenu.mbTransfer \
    -in .fMenu -anchor w -expand 0 -fill x -side left
  pack .fMenu.mbRec \
    -in .fMenu -anchor w -expand 0 -fill none -side top 
  pack .fEntry \
    -in . -anchor center -expand 0 -fill x -side top 
  grid .fEntry.lblNbr \
    -in .fEntry -column 0 -row 0 -columnspan 1 -rowspan 1 -sticky w
  grid .fEntry.entNbr \
    -in .fEntry -column 0 -row 1 -columnspan 1 -rowspan 1 
  grid .fEntry.lblDate \
    -in .fEntry -column 1 -row 0 -columnspan 1 -rowspan 1 -sticky w
  grid .fEntry.entDate \
    -in .fEntry -column 1 -row 1 -columnspan 1 -rowspan 1
  grid .fEntry.lblPayee \
    -in .fEntry -column 3 -row 0 -columnspan 1 -rowspan 1 -sticky w
  grid .fEntry.entPayee \
    -in .fEntry -column 3 -row 1 -columnspan 1 -rowspan 1
  grid .fEntry.lblDebit \
    -in .fEntry -column 4 -row 0 -columnspan 1 -rowspan 1 -sticky w
  grid .fEntry.entDebit \
    -in .fEntry -column 4 -row 1 -columnspan 1 -rowspan 1
  grid .fEntry.lblCredit \
    -in .fEntry -column 5 -row 0 -columnspan 1 -rowspan 1 -sticky w 
  grid .fEntry.entCredit \
    -in .fEntry -column 5 -row 1 -columnspan 1 -rowspan 1
  grid .fEntry.lbllstBalances \
    -in .fEntry -column 6 -row 0 -columnspan 1 -rowspan 1 -sticky w
  grid .fEntry.entlstBalances \
    -in .fEntry -column 6 -row 1 -columnspan 1 -rowspan 1 
  grid .fEntry.lblMemo \
    -in .fEntry -column 1 -row 2 -columnspan 1 -rowspan 1 -sticky e
  grid .fEntry.entMemo \
    -in .fEntry -column 3 -row 2 -columnspan 1 -rowspan 1 
  grid .fEntry.fSub1 \
    -in .fEntry -column 3 -row 3 -columnspan 1 -rowspan 1 
  pack .fEntry.fSub1.bDel \
    -in .fEntry.fSub1 -anchor center -expand 0 -fill none -side left 
  pack .fEntry.fSub1.bUpd \
    -in .fEntry.fSub1 -anchor center -expand 0 -fill x -side left 
  pack .fEntry.fSub1.bNew \
    -in .fEntry.fSub1 -anchor center -expand 0 -fill x -side left 
  grid .fEntry.fSub2 \
    -in .fEntry -column 4 -row 3 -columnspan 3 -rowspan 1 
  pack .fEntry.fSub2.bFirst \
    -in .fEntry.fSub2 -anchor center -expand 0 -fill none -side left 
  pack .fEntry.fSub2.bPref \
    -in .fEntry.fSub2 -anchor center -expand 0 -fill none -side left 
  pack .fEntry.fSub2.bNext \
    -in .fEntry.fSub2 -anchor center -expand 0 -fill none -side left 
  pack .fEntry.fSub2.bLast \
    -in .fEntry.fSub2 -anchor center -expand 0 -fill none -side left 
  grid .fEntry.cheRec \
    -in .fEntry -column 4 -row 2 -columnspan 2 -rowspan 1
  pack .register \
    -in . -anchor nw -expand 0 -fill both -side top
  pack .fStatus \
    -in . -anchor center -expand 0 -fill x -side bottom 
  pack .fStatus.lblStmtBal \
    -in .fStatus -anchor center -expand 0 -fill none -side left 
  pack .fStatus.entStmtBal \
    -in .fStatus -anchor center -expand 0 -fill none -side left 
  pack .fStatus.entActBal \
    -in .fStatus -anchor center -expand 0 -fill none -side right 
  pack .fStatus.lblActBal \
    -in .fStatus -anchor center -expand 0 -fill none -side right 
}

proc main {argc argv} {
  global wdgChkList state fname dirty 
  global lstBalances lstXacts lstPayees lstMemos
  global selectedRecord
  
  wm withdraw .

  set state view
  set fname ""
  set dirty 0
  set lstBalances {}
  set lstXacts {}
  set lstPayees {}
  set lstMemos {}

  if {$argc == 1} {
    set fname [lindex $argv 0]
    loading_splash_on [file tail $fname]
    MainWindow
    File_open
    loading_splash_off
    gotoSelectedRecord
  } else {
    tkbb_splash_on
    MainWindow
    after 4000 {
      tkbb_splash_off
    }
  }
}

main $argc $argv

